#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

###############################################################################
# Author: Thomas Wollner (tw@wollner-net.de)
###############################################################################
#
# 2021-08-20: rewritten for CMK 2.0 by thl-cmk[at]outlook[dot]com
# 2021-08-21: added more perfdata, added metrics file
# 2021-08-22: added WATO options, streamlined with cisco_bgp_peer
# 2021-08-29: moved static (longoutput) to inventory plugin
#             moved helper functions to utils/bgp_peer
# 2021-11-14: merged check function with cisco_bgp_peer
#             moved parse function to utils/bgp_peer
# 2021-04-02: rewritten bgp neighbor state handling (made configurable)
#
# ToDo: make check/discovery function the base for huawei_bgp_peer

# Example Agent Output:
# BGP4-MIB

# .1.3.6.1.2.1.15.3.1.1.192.168.254.2 = IpAddress: 192.168.254.2
# .1.3.6.1.2.1.15.3.1.2.192.168.254.2 = INTEGER: 3
# .1.3.6.1.2.1.15.3.1.3.192.168.254.2 = INTEGER: 2
# .1.3.6.1.2.1.15.3.1.4.192.168.254.2 = INTEGER: 4
# .1.3.6.1.2.1.15.3.1.5.192.168.254.2 = IpAddress: 0.0.0.0
# .1.3.6.1.2.1.15.3.1.7.192.168.254.2 = IpAddress: 192.168.254.2
# .1.3.6.1.2.1.15.3.1.9.192.168.254.2 = INTEGER: 65301
# .1.3.6.1.2.1.15.3.1.10.192.168.254.2 = Counter32: 0
# .1.3.6.1.2.1.15.3.1.11.192.168.254.2 = Counter32: 5
# .1.3.6.1.2.1.15.3.1.14.192.168.254.2 = Hex-STRING: 04 00
# .1.3.6.1.2.1.15.3.1.16.192.168.254.2 = Gauge32: 6586822
###############################################################################

import time
from typing import Dict, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    exists,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
    Metric,
    render,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)
from cmk.base.plugins.agent_based.utils.bgp_peer import (
    BgpPeer,
    bgp_get_peer_entry,
)


def parse_bgp_peer(string_table: StringTable) -> Optional[Dict[str, BgpPeer]]:
    peer_table = {}
    for entry in string_table:
        bgp_peer = bgp_get_peer_entry(entry)
        if bgp_peer:
            peer_table.update(bgp_peer)
    return peer_table


def discovery_bgp_peer(section: Dict[str, BgpPeer]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_bgp_peer(item, params, section: Dict[str, BgpPeer]) -> CheckResult:
    # default monitoring states for bgpPeerState
    neighborstate = {
        '1': 2,  # idle
        '2': 1,  # connect
        '3': 1,  # active
        '4': 1,  # opensent
        '5': 1,  # openconfirm
        '6': 0,  # established
    }

    neighborstate.update(params.get('neighborstate', neighborstate))  # update neighbor status with params

    peer_not_found_state = params['peernotfound']

    for bgp_connection, bgp_alias, not_found_state in params.get('peer_list', []):
        if item == bgp_connection:
            if bgp_alias != '':
                yield Result(state=State.OK, summary=f'[{bgp_alias}]', details=' ')
            peer_not_found_state = not_found_state

    # get item data
    try:
        peer = section[item]
    except KeyError:
        yield Result(state=State(peer_not_found_state), summary='Item not found in SNMP data')
        return

    if not peer.admin_state == 2:  # not start
        yield Result(state=State(params['admindown']), notice=f'Admin state: {peer.peer_statestr}')
    else:
        if peer.peer_state == 6:  # established
            yield from check_levels(
                value=peer.fsm_established_time,
                label='Uptime',
                levels_lower=params['minuptime'],
                render_func=render.timespan,
                metric_name='bgp_peer_fsmestablishedtime',
            )

        yield Result(state=State(neighborstate.get(str(peer.peer_state))), notice=f'Peer state: {peer.peer_statestr}')

    if peer.peer_unavail_reason != 0:  # huawei peer unavailable state
        yield Result(state=State.CRIT, notice=F'Peer unavailable reason: {peer.peer_unavail_reason_str}')

    acceptedprefixes = peer.accepted_prefixes
    prefixadminlimit = peer.prefix_admin_limit
    prefixthreshold = peer.prefix_threshold
    warnthreshold = None

    if prefixadminlimit is not None and prefixthreshold is not None:
        warnthreshold = int(prefixadminlimit / 100.0 * prefixthreshold)  # use float (100.0) to get xx.xx in division
    elif acceptedprefixes is not None:
        yield Result(
            state=State(params['noprefixlimit']),
            notice='Prefix limit/warn threshold: not configured on the device.',
        )
        warnthreshold = None

    if peer.admin_state == 2:  # no perfdata if admin shutdown
        if acceptedprefixes is not None:
            yield from check_levels(
                value=acceptedprefixes,
                metric_name='bgp_peer_acceptedprefixes',
                levels_upper=(warnthreshold, prefixadminlimit),
                label='Prefixes accepted',
                render_func=lambda v: f'{v}'
            )

        now_time = time.time()
        value_store = get_value_store()

        for entry in peer.metric_rate:
            key, value = entry
            try:
                value = get_rate(value_store, f'{key}', now_time, value, raise_overflow=True)
            except GetRateError:
                value = 0
            yield Metric(name=f'bgp_peer_{key}', value=value, boundaries=(0, None))

        for entry in peer.metric_count:
            key, value = entry
            yield Metric(name=f'bgp_peer_{key}', value=value, boundaries=(0, None))


register.snmp_section(
    name='bgp_peer',
    parse_function=parse_bgp_peer,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.15.3.1',  # BGP4-MIB::BgpPeerEntry
        oids=[
            '7',  # bgpPeerRemoteAddr
            '2',  # bgpPeerState
            '3',  # bgpPeerAdminStatus
            '10',  # bgpPeerInUpdates
            '11',  # bgpPeerOutUpdates
            '12',  # bgpPeerInTotalMessages
            '13',  # bgpPeerOutTotalMessages
            '15',  # bgpPeerFsmEstablishedTransitions
            '16',  # bgpPeerFsmEstablishedTime
            '24',  # bgpPeerInUpdateElapsedTime
        ]
    ),
    detect=exists('.1.3.6.1.2.1.15.3.1.1.*')
)

register.check_plugin(
    name='bgp_peer',
    service_name='BGP peer %s',
    discovery_function=discovery_bgp_peer,
    check_function=check_bgp_peer,
    check_default_parameters={
        'minuptime': (7200, 3600),
        'peer_list': [],
        'peernotfound': 2,
        'admindown': 1,
        'noprefixlimit': 1,
    },
    check_ruleset_name='bgp_peer',
)
