#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-08-29
#
# inventory of bgp peers
#
#

from typing import List
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    TableRow,
    exists,
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringTable,
    InventoryResult,
)
from cmk.base.plugins.agent_based.utils.bgp_peer import (
    ByteToHex,
    bgp_errors,
)


def parse_inv_bgp_peer(string_table: List[StringTable]):
    peers, base = string_table
    try:
        local_as, local_id = base[0]
    except (ValueError, IndexError):
        local_id = 'N/A'
        local_as = 'N/A'

    bgp_peers = []

    for entry in peers:
        try:
            remote_id,  version, local_addr, remote_addr, remote_as, last_error = entry
        except ValueError:
            return

        bgp_peers.append({
            'remote_addr': remote_addr,
            'remote_id': remote_id,
            'version': version,
            'local_addr': local_addr,
            'remote_as': remote_as,
            'local_as': local_as,
            'local_id': local_id,
            'bgp_type': 'iBGP' if local_as == remote_as else 'eBGP',
            'status_columns': {
                'last_error_code': ByteToHex(last_error),
                'last_error': bgp_errors(last_error),
            },
        })
    return bgp_peers


def inventory_bgp_peers(params, section) -> InventoryResult:
    path = ['networking', 'bgp_peers']

    for bgp_peer in section:

        key_columns = {'remote_addr': bgp_peer['remote_addr']}

        for key in key_columns.keys():
            bgp_peer.pop(key)

        status_columns = bgp_peer['status_columns']
        bgp_peer.pop('status_columns')

        yield TableRow(
            path=path,
            key_columns=key_columns,
            inventory_columns=bgp_peer,
            status_columns=status_columns
        )


register.snmp_section(
    name='inv_bgp_peer',
    parse_function=parse_inv_bgp_peer,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.2.1.15.3.1',  # BGP4-MIB::BgpPeerEntry
            oids=[
                '1',  # bgpPeerIdentifier
                '4',  # bgpPeerNegotiatedVersion
                '5',  # bgpPeerLocalAddr
                '7',  # bgpPeerRemoteAddr
                '9',  # bgpPeerRemoteAs
                '14',  # bgpPeerLastError
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.2.1.15',  # BGP4-MIB
            oids=[
                '2',  # bgpLocalAs
                '4',  # bgpIdentifier (local)
            ]
        )
    ],
    detect=exists('.1.3.6.1.2.1.15.3.1.1.*')
)

register.inventory_plugin(
    name='inv_bgp_peer',
    inventory_function=inventory_bgp_peers,
    inventory_default_parameters={
    },
    inventory_ruleset_name='inv_bgp_peer',
)
