#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-08-29
#
# include file, will be used with bgp_peer, inv_bgp_peer, cisco_bgp_peer, inv_cisco_bgp_peer
#
# 2022-04-17: added peer_unavail_reason/peer_unavail_reason_str for huawei bgp peers
#

import re
from typing import List, Tuple, Optional, Dict
from dataclasses import dataclass


@dataclass
class BgpPeer:
    peer_state: int
    peer_statestr: str
    admin_state: int
    admin_statestr: str
    fsm_established_time: int
    metric_rate: List[Tuple[str, int]]
    metric_count: List[Tuple[str, int]]
    prefix_admin_limit: Optional[int]
    prefix_threshold: Optional[int]
    prefix_clear_threshold: Optional[int]
    accepted_prefixes: Optional[int]
    peer_unavail_reason: Optional[int]
    peer_unavail_reason_str: Optional[str]


def sec2hr(seconds):
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    d, h = divmod(h, 24)
    if d > 0:
        return f'{d}d {h:02.0f}:{m:02.0f}:{s:02.0f}'
    else:
        return f'{h:02.0f}:{m:02.0f}:{s:02.0f}'


def bgp_peerstate(st):
    names = {
        0: 'none',
        1: 'idle',
        2: 'connect',
        3: 'active',
        4: 'opensnet',
        5: 'openconfirm',
        6: 'established'
    }
    return names.get(st, st)


def bgp_adminstate(st):
    names = {
        1: 'shutdown',
        2: 'enabled',
    }
    return names.get(st, st)


def ByteToHex(byteStr):
    return ''.join(['%02X ' % ord(x) for x in byteStr]).strip()


def bgp_errors(bytestring):
    byte1, byte2 = ByteToHex(bytestring).split()

    names = {}
    names[0] = {0: 'NO ERROR'}
    names[1] = {
        0: 'Message',
        2: 'Connection Not Synchronized',
        3: 'Bad Message Length',
        4: 'Bad Message Type',
    }
    names[2] = {
        0: 'OPEN',
        1: 'Unsupported Version Number',
        2: 'Bad Peer AS',
        3: 'Bad BGP Identifier',
        4: 'Unsupported Optional Parameter',
        5: 'Authentication Failure',
        6: 'Unacceptable Hold',
    }
    names[3] = {
        0: 'UPDATE',
        1: 'Malformed Attribute List',
        2: 'Unrecognized Well-known Attribute',
        3: 'Missing Well-known Attribute',
        4: 'Attribute Flags Error',
        5: 'Attribute Length Error',
        6: 'Invalid ORIGIN Attribute',
        7: 'AS Routing Loop',
        8: 'Invalid NEXT_HOP Attribute',
        9: 'Optional Attribute Error',
        10: 'Invalid Network Field',
        11: 'Malformed AS_PATH',
    }
    names[4] = {0: 'Hold Timer Expired', }

    names[5] = {0: 'Finite State Machine Error', }

    names[6] = {
        0: 'Administratively Shutdown',
        1: 'Max Prefix Reached',
        2: 'Peer Unconfigured',
        3: 'Administratively Reset',
        4: 'Connection Rejected',
        5: 'Other Configuration Change',
    }
    return names[int(byte1, 16)].get(int(byte2, 16))


def bgp_render_ipv4_address(bytestring):
    return '.'.join([f'{ord(m)}' for m in bytestring])


def bgp_shorten_ipv6_adress(address):
    address = address.split(':')
    span = 2
    address = [''.join(address[i:i + span]) for i in range(0, len(address), span)]
    for m in range(0, len(address)):
        address[m] = re.sub(r'^0{1,3}', r'', address[m])
    address = ':'.join(address)
    zeros = ':0:0:0:0:0:0:'
    while not zeros == '':
        if zeros in address:
            address = re.sub(r'%s' % zeros, r'::', address)
            zeros = ''
        else:
            zeros = zeros[:-2]
    return address


def bgp_render_ipv6_address(bytestring):
    address = ":".join(["%02s" % hex(ord(m))[2:] for m in bytestring]).replace(' ', '0').upper()
    address = bgp_shorten_ipv6_adress(address)

    return address


def bgp_render_ip_address(bytestring):
    if len(bytestring) == 4:
        return bgp_render_ipv4_address(bytestring)
    elif len(bytestring) == 16:
        return bgp_render_ipv6_address(bytestring)
    else:
        return ''


def bgp_get_peer(OID_END):
    # returns peer address string from OID_END
    # u'1.4.217.119.208.34.1.1' --> 217.119.208.34
    # u'2.20.42.5.87.192.0.0.255.255.0.0.0.0.0.0.0.17.2.1' -->  42.5.87.192.0.0.255.255.0.0.0.0.0.0.0.17
    peer_ip = ''
    OID_END = OID_END.split('.')
    if int(OID_END[1]) == 4:  # length of ip address
        peer_ip = '.'.join(OID_END[2:6])  # ipv4 address
    elif int(OID_END[1]) == 16:  # ipv6 address
        peer_ip = ':'.join('%02s' % hex(int(m))[2:] for m in OID_END[2:18]).replace(' ', '0').upper()
        peer_ip = bgp_shorten_ipv6_adress(peer_ip)

    return peer_ip


def bgp_get_peer_entry(peer: List) -> Optional[Dict[str, BgpPeer]]:
    try:
        remote_addr, peer_state, admin_state, in_updates, out_updates, in_messages, out_messages, \
        fsm_established_transitions, fsm_established_time, in_update_elapsed_time = peer
    except ValueError:
        return

    bgp_peer = BgpPeer(
        peer_state=int(peer_state),
        peer_statestr=bgp_peerstate(int(peer_state)),
        admin_state=int(admin_state),
        admin_statestr=bgp_adminstate(int(admin_state)),
        fsm_established_time=int(fsm_established_time),
        metric_count=[],
        metric_rate=[],
        prefix_admin_limit=None,
        prefix_threshold=None,
        prefix_clear_threshold=None,
        accepted_prefixes=None,
        peer_unavail_reason=0,
        peer_unavail_reason_str='',
    )

    for key, value in [
        ('inupdates', in_updates),
        ('outupdates', out_updates),
        ('intotalmessages', in_messages),
        ('outtotalmessages', out_messages),
    ]:
        try:
            bgp_peer.metric_rate.append((key, int(value)))
        except (ValueError, TypeError):
            pass

    for key, value in [
        ('fsmestablishedtransitions', fsm_established_transitions),
        ('inupdateelapsedtime', in_update_elapsed_time),
    ]:
        try:
            bgp_peer.metric_count.append((key, int(value)))
        except (ValueError, TypeError):
            pass

    return {remote_addr: bgp_peer}
